/*
$Id: open.c,v 1.4 1995/11/05 18:00:18 ssmore Exp $

File Name : open.c
Author    : ssmore

Entry Point(s) :

(1) PASSION_open   prototype in passion.h

Modification History :

Module Description                                      Date      Author
------------------------------------------------------------------------------
(1) PASSION_open                                        2/22/95    ssmore
(1) PASSION_open                                        2/27/95    ssmore

Notes :

(1) Modifications to the entry points interface should be reflected in the 
    prototypes declarations.
*/

#ifndef PASSION_OPEN_C
#define PASSION_OPEN_C

/*#define DEBUG*/

#ifdef DEBUG
#include <stdio.h>
#endif

#include <fcntl.h>

#include "pfile.h"

/*
  Name : PASSION_open

  Function : This function is used to open the Local Array File (LAF) or the 
	     global file.

  Parameters : 
    Input :
      FileName - pointer to a ASCIIZ string, it is the name of the LAF/global
		 file to be opened
    Output :
      none
    Input/Ooutput :
      none

  Return Value : 
    Pointer to PFILE structure, if successful
    (PFILE *)0, if not successful

  Algorithm :

    if file name is not specified
      return error 
    endif

    allocate space for the parallel file data structure (PFILE)
    if out of memory space
      return error
    endif

    open the file for read/write
    if not successful
      return error
    endif

    put the file descriptor in the PFILE structure

    return the pointer to the PFILE structure

  Side Effects :
    (1) Allocates a PFILE structure that has to be de-allocated when the file is
        closed.

  Shortcomings :
    (1) can not create files.
    (2) treats LAF and global file in the same way.

  Modifications :

  Description                                       Date          Author
  ------------------------------------------------------------------------------
  (1) Ported successfully to PIOFS without any      2/22/95       ssmore
	    modifications.
  (2) Support for multiple dimensions               2/27/95       ssmore
			No changes made to the code
*/
PFILE *PASSION_open(char *FileName, unsigned int HeaderSize)
{
PFILE *PFilePtr;  /* pointer to the PFILE buffer, is the return value, if all 
		     goes well */
LAF_File fd;           /* system file descriptor of the file opened */
SHARE_PFILE  *ShPFilePtr;
#ifdef PASSION_TRACE
int trace_fd;
char trace_fname[128];
#endif

  if (FileName == (char *)0) 
  {
#ifdef DEBUG
    fprintf(stderr,"PASSION : (open) No filename specified\n");
#endif
	return (PFILE *)0;
  }
  if (FileName[0] == (char)0) 
  {
#ifdef DEBUG
    fprintf(stderr,"PASSION : (open) No filename specified\n");
#endif
	return (PFILE *)0;
  }

  PFilePtr = (PFILE *)malloc(sizeof(PFILE));
  ShPFilePtr = (SHARE_PFILE *)malloc(sizeof(SHARE_PFILE));
  if (PFilePtr == (PFILE *)0 || ShPFilePtr == (SHARE_PFILE *)0)
  {
#ifdef DEBUG
    fprintf(stderr,"PASSION : (open) Not enough memory\n");
#endif
    return (PFILE *)0;
  }

  /*
    The open flag (O_RDWR) should be specified, dynamically by the user.
  */
#ifdef HAVE_MPIO
  {
  char prefix[500];
  strcpy(prefix,"ufs:");
  strcpy(prefix+4, FileName);
  MPI_File_open(MPI_COMM_SELF, prefix, MPI_MODE_RDWR , MPI_INFO_NULL, &fd);
   /* no error handle */
  if (fd == NULL) { 
    fprintf(stderr,"PASSION : (open) System Error\n");
    return (PFILE *)0;
  }
  }
#else		/* HAVE_MPIO */
  fd = open(FileName, O_RDWR);
  if (fd == -1)
  {
#ifdef DEBUG
    fprintf(stderr,"PASSION : (open) System Error\n");
#endif
    free((char *)PFilePtr);
#ifdef DEBUG
    perror("open");
#endif
    return (PFILE *)0;
  }
#endif           /* HAVE_MPIO */

  PFilePtr->share = ShPFilePtr;
  ShPFilePtr->filename = FileName;
  ShPFilePtr->Fd = fd;
  PFilePtr->HeaderSize = HeaderSize;
  ShPFilePtr->Status = PASSION_OLD;

#ifdef PASSION_TRACE
  sprintf(trace_fname, "%s.trace", FileName);
  trace_fd = open(trace_fname, O_WRONLY | O_CREAT | O_TRUNC, 0600);
  if (trace_fd == -1)
  {
	fprintf(stderr, "Could not open the trace file %s\n", trace_fname);
  }
  PFilePtr->trace_fd = trace_fd;
#endif
  return PFilePtr;
}


/* PFILE copy function */
PFILE *CopyPfile(PFILE *pfile)
{
PFILE *PFilePtr; 
int i;

  PFilePtr = (PFILE *)malloc(sizeof(PFILE));
  PFilePtr->share = pfile->share;
  PFilePtr->HeaderSize = pfile->HeaderSize;
  for (i=0; i<10; i++) PFilePtr->ocla_size[i] = pfile->ocla_size[i];
  
#ifdef PASSION_TRACE
  PFilePtr->trace_fd = pfile->trace_fd;
#endif

  return PFilePtr;
}

#endif  /* PASSION_OPEN_C */
